/*
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************

 JuickPlugin
     Copyright (c) 2010 by Alexey Ryabov <6006l1k@gmail.com>
*/

#include "jplugin.h"

bool JuickPlugin::init(PluginSystemInterface *plugin_system)
{
	qRegisterMetaType<TreeModelItem>("TreeModelItem");

	PluginInterface::init(plugin_system);
	m_plugin_icon = QIcon(":/icons/plugin.png");
	m_plugin_system = plugin_system;

	m_event_msgin = m_plugin_system->registerEventHandler("Core/ChatWindow/ReceiveLevel3", this);

	// It's not important, because setProfileName will be called at first
	m_user_bold = m_user_italic = m_msg_bold = m_msg_italic = m_msg_underline = false;

	return true;
}

void JuickPlugin::release()
{

}

void JuickPlugin::processEvent (Event &event)
{
	if (event.id == m_event_msgin && event.size() >= 2) {
		TreeModelItem item = *(TreeModelItem *)(event.args.at(0));

		if (item.m_item_name != "juick@juick.com" && item.m_item_name != "jubo@nologin.ru")
			return;

		QString *text = (QString *)event.args.at(1);
		QString msg = *text;
		static QRegExp jpost("(#\\d+\\s|#\\d+/\\d+)");
		static QRegExp jusers("(@[a-zA-Z0-9-_@\\.]+)\\b");
		static QRegExp junderline("(\\s?)(_(\\S+)_)(\\s?)");
		static QRegExp jbold("(\\s?)(\\*(\\S+)\\*)(\\s?)");
		static QRegExp jitalic("(\\s+)(/(\\S+)/)(\\s+)");

		QString user_style = "color:"+m_user_color.name()+";";
		QString msg_style = "color:"+m_msg_color.name()+";";
		if (m_user_bold) user_style += "font-weight:bold;";
		if (m_user_italic) user_style += "font-style:italic;";
		if (m_user_underline) user_style += "text-decoration:underline;";
		if (m_msg_bold) msg_style += "font-weight:bold;";
		if (m_msg_italic) msg_style += "font-style:italic;";
		if (m_msg_underline) msg_style += "text-decoration:underline;";

		QString toReplace;

		int pos = 0;
		while ((pos = jpost.indexIn((*text), pos)) != -1) {
			toReplace = "<span style=\""+msg_style+"\">"+jpost.cap(1)+"</span>";
			text->replace(pos, jpost.cap(1).length(), toReplace);
			pos += toReplace.length();
		}
		pos = 0;
		while ((pos = jusers.indexIn((*text), pos)) != -1) {
			toReplace = "<span style=\""+user_style+"\">"+jusers.cap(1)+"</span>";
			text->replace(pos, jusers.cap(1).length(), toReplace);
			pos += toReplace.length();
		}
		// Underlines
		pos = 0;
		while ((pos = junderline.indexIn(msg, pos)) != -1) {
			text->replace(junderline.cap(2), "<u>"+junderline.cap(3)+"</u>");
			pos += junderline.matchedLength();
		}
		// Bolds
		pos = 0;
		while ((pos = jbold.indexIn(msg, pos)) != -1) {
			text->replace(jbold.cap(2), "<b>"+jbold.cap(3)+"</b>");
			pos += jbold.matchedLength();
		}
		// Italics
		pos = 0;
		while ((pos = jitalic.indexIn(msg, pos)) != -1) {
			text->replace(jitalic.cap(2), "<i>"+jitalic.cap(3)+"</i>");
			pos += jitalic.matchedLength();
		}
		//text->insert(0, "<div style=\"background-color:#EEEEDF;\">");
		//text->insert(text->length(), "</div>");
	}
}

void JuickPlugin::saveSettings()
{
	m_user_bold = m_button_user_bold->isChecked();
	m_user_italic = m_button_user_italic->isChecked();
	m_user_underline = m_button_user_underline->isChecked();
	m_msg_bold = m_button_msg_bold->isChecked();
	m_msg_italic = m_button_msg_italic->isChecked();
	m_msg_underline = m_button_msg_underline->isChecked();
	m_user_color = m_button_user_color->property("sett_color").value<QColor>();
	m_msg_color = m_button_msg_color->property("sett_color").value<QColor>();

	QSettings settings(QSettings::IniFormat, QSettings::UserScope, "qutim/qutim."+m_profile_name, "plugin_juick");
	settings.setValue("user_bold", m_button_user_bold->isChecked());
	settings.setValue("user_italic", m_button_user_italic->isChecked());
	settings.setValue("user_underline", m_button_user_underline->isChecked());
	settings.setValue("msg_bold", m_button_msg_bold->isChecked());
	settings.setValue("msg_italic", m_button_msg_italic->isChecked());
	settings.setValue("msg_underline", m_button_msg_underline->isChecked());
	settings.setValue("user_color", m_button_user_color->property("sett_color").value<QString>());
	settings.setValue("msg_color", m_button_msg_color->property("sett_color").value<QString>());
}

void JuickPlugin::loadSettings()
{
	QSettings settings(QSettings::IniFormat, QSettings::UserScope, "qutim/qutim."+m_profile_name, "plugin_juick");

	m_user_bold = settings.value("user_bold", true).toBool();
	m_user_italic = settings.value("user_italic", false).toBool();
	m_user_underline = settings.value("user_underline", false).toBool();

	m_msg_bold = settings.value("msg_bold", false).toBool();
	m_msg_italic = settings.value("msg_italic", false).toBool();
	m_msg_underline = settings.value("msg_underline", false).toBool();

	m_user_color = settings.value("user_color", QColor(0, 0, 0)).toString();
	m_msg_color = settings.value("msg_color", QColor(0, 128, 0)).toString();
}

QWidget *JuickPlugin::settingsWidget()
{
	// It looks like Psi+ style, I'm too lazy to fix it, but it's annoying
	m_settings_form = new QWidget;
	QVBoxLayout *vbox = new QVBoxLayout(m_settings_form);
	QGroupBox * frame = new QGroupBox(tr("Juick plugin settings"), m_settings_form);
	QGridLayout *layout = new QGridLayout(frame);
	layout->addWidget(new QLabel("@username", m_settings_form), 1, 0);
	layout->addWidget(new QLabel("#msg_id", m_settings_form), 2, 0);
	layout->addWidget(new QLabel(tr("bold"), m_settings_form), 0, 1);
	layout->addWidget(new QLabel(tr("italic"), m_settings_form), 0, 2);
	layout->addWidget(new QLabel(tr("underline"), m_settings_form), 0, 3);
	layout->addWidget(new QLabel(tr("color"), m_settings_form), 0, 4);
	//user
	m_button_user_bold = new QCheckBox(m_settings_form);
	m_button_user_italic = new QCheckBox(m_settings_form);
	m_button_user_underline = new QCheckBox(m_settings_form);
	m_button_user_color = new QToolButton(m_settings_form);
	layout->addWidget(m_button_user_bold, 1, 1);
	layout->addWidget(m_button_user_italic, 1, 2);
	layout->addWidget(m_button_user_underline, 1, 3);
	layout->addWidget(m_button_user_color, 1, 4);
	//msg_id
	m_button_msg_bold = new QCheckBox(m_settings_form);
	m_button_msg_italic = new QCheckBox(m_settings_form);
	m_button_msg_underline = new QCheckBox(m_settings_form);
	m_button_msg_color = new QToolButton(m_settings_form);
	layout->addWidget(m_button_msg_bold, 2, 1);
	layout->addWidget(m_button_msg_italic, 2, 2);
	layout->addWidget(m_button_msg_underline, 2, 3);
	layout->addWidget(m_button_msg_color, 2, 4);

	QButtonGroup *grColor = new QButtonGroup(m_settings_form);
	grColor->addButton(m_button_user_color);
	grColor->addButton(m_button_msg_color);

	m_button_user_color->setStyleSheet(QString("background-color: %1;").arg(m_user_color.name()));
	m_button_msg_color->setStyleSheet(QString("background-color: %1;").arg(m_msg_color.name()));
	m_button_user_color->setProperty("sett_color", m_user_color);
	m_button_msg_color->setProperty("sett_color", m_msg_color);
	m_button_user_bold->setChecked(m_user_bold);
	m_button_user_italic->setChecked(m_user_italic);
	m_button_user_underline->setChecked(m_user_underline);
	m_button_msg_bold->setChecked(m_msg_bold);
	m_button_msg_italic->setChecked(m_msg_italic);
	m_button_msg_underline->setChecked(m_msg_underline);
	connect(grColor, SIGNAL(buttonClicked(QAbstractButton*)), SLOT(chooseColor(QAbstractButton*)));
	layout->setAlignment(Qt::AlignTop);
	frame->setLayout(layout);
	vbox->addWidget(frame);
	return m_settings_form;
}

void JuickPlugin::removeSettingsWidget()
{
	delete m_settings_form;
}

void JuickPlugin::setProfileName(const QString &profile_name)
{
	m_profile_name = profile_name;
	loadSettings();
}

QString JuickPlugin::name()
{
	return "Juick";
}

QString JuickPlugin::description()
{
	return "Juick Plugin";
}

QIcon *JuickPlugin::icon()
{
	return &m_plugin_icon;
}

QString JuickPlugin::type()
{
	return "other";
}

void JuickPlugin::chooseColor(QAbstractButton* button)
{
	QColor c;
	c = button->property("sett_color").value<QColor>();
	c = QColorDialog::getColor(c, new QWidget());
	if(c.isValid()) {
		button->setProperty("sett_color", c);
		button->setStyleSheet(QString("background-color: %1").arg(c.name()));
	}
}

Q_EXPORT_PLUGIN2 (juickplugin, JuickPlugin);
