#include "statusesedit.h"
#include "ui_statusesedit.h"
#include "../coresrc/MRIMUtils.h"
#include "../coresrc/MRIMClient.h"
#include <QRegExpValidator>

statusesEdit::statusesEdit(MRIMClient *pClnt, QWidget *parent) :
    QWidget(parent),
    m_ui(new Ui::statusesEdit),
    pClient(pClnt)
{
    m_ui->setupUi(this);
    QString statusPref = "status_";

    for (int i = 4; i < 54; i++)
    {
        QIcon icon = Status::GetIcon(statusPref+QString::number(i));

        m_ui->icon1->addItem(icon, "", i);
        m_ui->icon2->addItem(icon, "", i);
        m_ui->icon3->addItem(icon, "", i);
        m_ui->icon4->addItem(icon, "", i);
        m_ui->icon5->addItem(icon, "", i);
        m_ui->icon6->addItem(icon, "", i);
    }

    int line = 1;
    QList<QAction*> statusesList = pClient->getStatusGroup()->actions();
    QRegExp rx("^status_[\\d]*$");
    QRegExpValidator validator(rx, NULL);
    QString status;
    QComboBox *currIconLine = 0;
    QLineEdit *currTextLine = 0;

    foreach (QAction *statusAction, statusesList)
    {
        status = statusAction->data().toString();
        int pos = 0;
        if (validator.validate(status, pos) != QValidator::Acceptable)
                continue;

        if (currIconLine = getIconBox(line))
        {
            status.remove("status_");
            currIconLine->setCurrentIndex(status.toInt() - 4);
            currIconLine = 0;
        }

        if (currTextLine = getStatusEdit(line))
        {
            currTextLine->setText(statusAction->text());
            currTextLine = 0;
        }
        line++;
    }
}

statusesEdit::~statusesEdit()
{
    delete m_ui;
}

void statusesEdit::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void statusesEdit::on_okButton_clicked()
{
    QSettings acc_settings(QSettings::defaultFormat(), QSettings::UserScope, "qutim/qutim."+PluginSystemImpl()->Profile()+"/mrim."+pClient->getAccountName(), "savedstatuses");
    QStringList statusesList = acc_settings.childGroups();
    QComboBox *currIconLine = 0;
    quint32 line = 1;

    while ( currIconLine = getIconBox(line) )
    {
        acc_settings.beginGroup(QString::number(line));
        acc_settings.setValue("status", STATUS_USER_DEFINED);
        acc_settings.setValue("status_id", QString("status_%1").arg(QString::number(currIconLine->currentIndex() + 4)));
        acc_settings.setValue("statusTitle", getStatusEdit(line)->text());
        acc_settings.endGroup();
        line++;
    }

    pClient->UpdateStatuses();
    close();
}

void statusesEdit::on_cancelButton_clicked()
{
    close();
}

QComboBox *statusesEdit::getIconBox(int index)
{
    return findChild<QComboBox*>(QString("icon%1").arg(index));
}

QLineEdit *statusesEdit::getStatusEdit(int index)
{
    return findChild<QLineEdit*>(QString("text%1").arg(index));
}
