/*
    emoticonMenu

    Copyright (c) 2008 by Rustam Chakin <qutim.develop@gmail.com>
				  2009 by Ruslan Nigmatullin <euroelessar@gmail.com>

 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
*/


#include "chatemoticonbar.h"
#include <QVector>

ChatEmoticonBar::ChatEmoticonBar(QWidget *parent)
	: QScrollArea(parent)
{
	m_widget = 0;
	m_grid_layout = 0;
	setFrameStyle(QFrame::NoFrame);
	setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        timer = new QTimer( this );
        connect( timer, SIGNAL(timeout()), this, SLOT(hide()) );
}

ChatEmoticonBar::~ChatEmoticonBar()
{
	m_widget = 0;
	clearList();
}

void ChatEmoticonBar::setAutoclose(bool autoclose, int time)
{
    m_autoclose=autoclose;
    m_emoclosetime = time;
}

void ChatEmoticonBar::resetTimer()
{
        timer->stop();
        timer->start( m_emoclosetime * 1000 );
}

void ChatEmoticonBar::setEmoticons(QHash<QString, QStringList> list)
{
	clearList();
	m_widget = new QWidget;
        //m_grid_layout = new QGridLayout(m_widget);
        //m_grid_layout->setSpacing(1);
        //m_widget->setLayout(m_grid_layout);
	int max_len_size = 0;

	m_desktop_geometry = QSize();

	QHash<int, QString> emotOrder;
	{
		QHashIterator<QString, QStringList> i(list);
		while (i.hasNext())
		{
			i.next();
			QString key = i.key();
			list.remove(key);
			int index = key.indexOf("|");
			int num = key.mid(0, index).toInt();
			key.remove(0, index+1);
			emotOrder.insert(num, key);
			list.insert(key, i.value());
		}
	}

        QHashIterator<int, QString> i(emotOrder);
	while (i.hasNext())
	{
		i.next();
		QStringList values = list.value(i.value());
		if(!values.size())
			continue;

		movieLabel *label = new movieLabel;
		labelList << label;
		QMovie *movie = new QMovie(i.value());
		movieList << movie;
		label->setMovie(movie);
		movie->setCacheMode(QMovie::CacheAll);
		movie->start();
                QSize size = movie->currentPixmap().size();
                label->setMinimumSize(size);
                sizeList << size;
		label->setToolTip(values.first());
                connect(label, SIGNAL(sendMovieTip(const QString &)), this, SIGNAL(insertSmile(const QString &)));
		movie->stop();
	}

}

void ChatEmoticonBar::clearList()
{
//	foreach(movieLabel *l, labelList)
//		delete l;
	delete m_widget;
	m_widget = 0;
	
	sizeList.clear();
	qDeleteAll(labelList);
	labelList.clear();
		
//	foreach(QMovie *m, movieList)
//		delete m;
	qDeleteAll(movieList);
	movieList.clear();
}


void ChatEmoticonBar::hideEvent(QHideEvent *e)
{
	foreach(QMovie *m, movieList)
		m->stop();
        timer->stop();
        emit m_hide();
//	clearList();
	QWidget::hideEvent(e);
}

void ChatEmoticonBar::showEvent(QShowEvent *e)
{
//	ensureGeometry();
//	setEmoticons(emotList, emotPath);
            foreach(QMovie *m, movieList)
            {
                    m->setCacheMode(QMovie::CacheAll);
                    m->start();
            }

        if(m_autoclose) timer->start( m_emoclosetime * 1000 ); //
	QWidget::showEvent(e);
}

void ChatEmoticonBar::ensureGeometry()
{
        if(sizeList.isEmpty() || m_grid_layout!=NULL)
		return;
        int hcnt=sizeList.count()>>1; //all smiles into 2 lines
        int k=0,l=0,col=0;
        //Create vlayer and 2 hlayers
        m_grid_layout = new QVBoxLayout(m_widget);
        m_grid_layout->setSpacing(1);
        QHBoxLayout *lsmiles1 = new QHBoxLayout();
        QHBoxLayout *lsmiles2 = new QHBoxLayout();
        m_grid_layout->addLayout(lsmiles1);
        m_grid_layout->addLayout(lsmiles2);
        m_widget->setLayout(m_grid_layout);

        int height[2] = {0,0};

        for(int i = 0; i < labelList.size(); i++)
        {
                if((i>hcnt) && (k==0))
                {
                    k=1;

                }
                if(height[k]<sizeList[i].height())height[k]=sizeList[i].height();
                if(k==0)
                 lsmiles1->addWidget(labelList[i]);
                else
                 lsmiles2->addWidget(labelList[i]);

        }


        setWidget(m_widget);
        resize(0,m_widget->geometry().height()+16);
        setMinimumSize(size());


}


void ChatEmoticonBar::wheelEvent(QWheelEvent *e)
{
    QWheelEvent me( e->pos(), e->delta(), e->buttons(), e->modifiers(), Qt::Horizontal );
    //QScrollArea::wheelEvent(&me);
    QApplication::sendEvent(this->horizontalScrollBar(), &me);
}

